/*
 * Copyright (c) 1994 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Dlg/dlgcov.h>
#include <Dlg/dlgdb.h>
#include <Dlg/dlgfclass.h>
#include <Dlg/dlglib.h>

#include <TopoFace/topoedge.h>
#include <TopoFace/topoface.h>
#include <TopoFace/toponode.h>

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>

/*****************************************************************************/

DlgCoverage::DlgCoverage (DlgLibrary* lib, const char* name) 
: MapCoverage(lib, name) {
    _path = nil;
    _desc = NULL;
}

DlgCoverage::~DlgCoverage () {
    delete _desc;
}

const char* DlgCoverage::path() const {
    if (_library && !_path) {
	const char* libpath = _library->path();
        if (((DlgDatabase*)(this->library()->database()))->Is100K()) {
	    DIR* dir;
	    struct dirent* entry;
	    dir = opendir(libpath);
	    if (dir) {
	        while ((entry = readdir(dir)) != NULL) {
		    if ((strcmp(entry->d_name,".") != 0) &&
		        (strcmp(entry->d_name,"..") != 0) &&
		        (strstr(entry->d_name,_name))) {
			    delete _path;
			    ((DlgCoverage*)this)->_path = new char[strlen(libpath)
					 + strlen(entry->d_name) + 2];
			    char* path = ((DlgCoverage*)this)->_path;
			    strcpy( path, libpath );
			    strcat( path, "/");
			    strcat( path, entry->d_name);
			    break;
		        }
	        }
	    }
	    closedir(dir);
        }
        else {
	    struct stat buf;
	    delete _path;
	    ((DlgCoverage*)this)->_path = new char[strlen(libpath) + strlen(_name) + 15];
	    char* path = ((DlgCoverage*)this)->_path;
	    strcpy( path, libpath );
	    strcat( path, "/s" );
	    strcat( path, libpath+strlen(libpath)-2 );
	    strcat( path, "_" );
	    strcat( path, _name );
	    strcat( path, ".lgs" );
	    if (stat(path, &buf) != 0)
	        strcat(path, ".Z");
	    if (stat(path, &buf) != 0) {
	        path[strlen(path)-2] = '\0';
	        strcat(path, ".gz");
	    }
	    if (stat(path, &buf) != 0)
	        path[strlen(path)-3] = '\0';
        }
    }
    return _path;
}

DlgFeatureClass* DlgCoverage::feature_class(unsigned fclsnum) const {
    return (DlgFeatureClass*)MapCoverage::feature_class(fclsnum);
}

DlgFeatureClass* DlgCoverage::feature_class(const char* name) const {
    return (DlgFeatureClass*)MapCoverage::feature_class(name);
}

char** DlgCoverage::feature_class_names (int& nfcls) const {
    if (((DlgDatabase*)library()->database())->Is100K()) {
	nfcls = 0;
	DIR* dir;
	struct dirent* entry;
	dir = opendir(path());
	while ((entry = readdir(dir)) != NULL)
	  if ((strcmp(entry->d_name,".") != 0) &&
	      (strcmp(entry->d_name,"..") != 0))
	    nfcls++;
	char **fclasses = new char*[nfcls];
	int fi = 0;
	dir = opendir(path());
	while ((entry = readdir(dir)) != NULL)
	  if ((strcmp(entry->d_name,".") != 0) &&
	      (strcmp(entry->d_name,"..") != 0))
	    fclasses[fi++] = strdup(entry->d_name);
	return fclasses;
    }
    else {
	nfcls = 1;
	char **fclasses = new char*[nfcls];
	if (strcmp(name(), "ab") == 0 ||
	    strcmp(name(), "pb") == 0 ||
	    strstr(name(), "wb")) 
	    fclasses[0] = strdup("area");
	else
	    fclasses[0] = strdup("line");
	return fclasses;
    }
}

MapFeatureClass* DlgCoverage::make_feature_class(const char* name) { 
    return new DlgFeatureClass(this,name);
}

void DlgCoverage::description(char* d) {
  delete _desc;
  _desc = strdup(d);
}

const char* DlgCoverage::description() const {
    if (!_desc) {
	if (((DlgDatabase*)library()->database())->Is100K()) {
	  DIR* dir;
	  struct dirent* entry;
	  dir = opendir(path());
	  if (dir) {
	    while ((entry = readdir(dir)) != NULL)
	      if ((strcmp(entry->d_name,".") != 0) &&
		  (strcmp(entry->d_name,"..") != 0)) {
		char pathname[256];
		strcpy(pathname, path());
		strcat(pathname, "/");
		strcat(pathname, entry->d_name);
		FILE* fptr = fopen(pathname, "r");
		if (fptr) {
		  char buffer[82];
		  for (int recnum=0; recnum<15; recnum++)
		    fgets (buffer, 82, fptr);
		  char*& desc = ((DlgCoverage *)this)->_desc;
		  desc = new char[21];
		  strncpy (desc, buffer, 20);
		  desc[20] = '\0';
		  fclose(fptr);
		  return _desc;
		}
	      }
	  }
	}
	else {
	    char* shcmd = new char[strlen("gunzip -c ") + 1 + strlen(path())];
	    if (strstr(path(), ".Z"))
		strcpy( shcmd, "zcat ");
	    else if (strstr(path(), ".gz"))
		strcpy( shcmd, "gunzip -c ");
	    else
		strcpy( shcmd, "cat ");
	    strcat( shcmd, path());

	    FILE* fptr  = popen( shcmd, "r");
	    if (!fptr) {
		delete shcmd;
		return NULL;
	    }
	    char buffer[145];
	    buffer[144] = '\0';
	    int recnum;

	    /* "A" records */
	    for (recnum=0; recnum<6; recnum++)
		fread (buffer, 1, 144, fptr);
	    
	    /* "B" records */
	    for (recnum=0; recnum<2; recnum++)
		fread (buffer, 1, 144, fptr);
	    
	    /* "C1" record */
	    fread (buffer, 1, 144, fptr);
	    int num_categories = atoi(buffer);
	    
	    /* "C2" record */
	    fread (buffer, 1, 144, fptr);
	    char*& desc = ((DlgCoverage *)this)->_desc;
	    desc = new char[21];
	    strncpy (desc, buffer, 20);
	    desc[20] = '\0';

	    do { fread (buffer, 1, 144, fptr); } while (!feof(fptr));

	    pclose(fptr);
	    delete shcmd;
	    return _desc;
	}
    }
    else
	return _desc;
}

