/*
 * Copyright (c) 1996, 1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef MAKEDEPEND
#include <vtkFloatPoints.h>
#include <vtkFloatScalars.h>
#include <vtkPolyData.h>
#endif

#include <stdio.h>
#include <string.h>

int main(int argc, char* argv[]) {
    if (argc != 3) {
	printf("usage: smf2vtk smffile vtkfile\n");
	return(-1);
    }
    vtkPolyData* polydata = vtkPolyData::New();
    vtkCellArray* cells = vtkCellArray::New();
    vtkFloatPoints* fltpoints = NULL;
    fltpoints = vtkFloatPoints::New();
    float fltp[3];
    int intp[3];
    FILE* smffile = fopen(argv[1], "r");
    if (smffile) {
      char linebuf[255];
      while (!feof(smffile)) {
	fgets(linebuf, 255, smffile);
	if (feof(smffile)) continue;
	if (linebuf[0] == '#') continue;
	if (linebuf[0] == 'v') {
	  sscanf(&linebuf[1], "%f %f %f", &fltp[0], &fltp[1], &fltp[2]);
	  fltpoints->InsertNextPoint(fltp);
	}
	else if (linebuf[0] == 'f') {
	  sscanf(&linebuf[1], "%d %d %d", &intp[0], &intp[1], &intp[2]);
	  intp[0]--;
	  intp[1]--;
	  intp[2]--;
	  cells->InsertNextCell(3, intp);
	}
      }
    }
    else {
      printf("couldn't open %s\n", argv[1]);
      return(-1);
    }
    polydata->SetPoints(fltpoints);
    fltpoints->Delete();
    polydata->SetPolys(cells);
    cells->Delete();

    int npt = polydata->GetNumberOfPoints();
    vtkFloatScalars* sc = vtkFloatScalars::New();
    sc->Allocate(npt);
    float point[3];
    for (int pti = 0; pti < npt; pti++) {
      polydata->GetPoint(pti, point);
      sc->InsertNextScalar(point[2]);
    }
    polydata->GetPointData()->SetScalars(sc);
    sc->Delete();

    float bounds[6];
    polydata->GetBounds(bounds);
    char bdattrs[256];
    sprintf(bdattrs, "%.4f %.4f %.4f %.4f %.4f %.4f",
	    bounds[0], bounds[1], bounds[2], bounds[3],
	    bounds[4], bounds[5]);

    FILE* fp = fopen(argv[2], "w");
    if (fp) {
      fprintf(fp, "# vtk DataFile Version 2.0\n");
      fprintf(fp, "%s\n", bdattrs);
      fprintf(fp, "ASCII\nDATASET POLYDATA\n");
      fprintf(fp, "POINTS %d float\n", npt);
      float point[3];
      for (int pti = 0; pti < npt; pti++) {
	polydata->GetPoint(pti, point);
	fprintf(fp, "%f %f %f ", point[0], point[1], point[2]);
      }
      int ncells = polydata->GetPolys()->GetNumberOfCells();
      int size = polydata->GetPolys()->GetNumberOfConnectivityEntries();
      fprintf(fp, "\nPOLYGONS %d %d\n", ncells, size);
      int npts, *pts, pj;
      for (polydata->GetPolys()->InitTraversal();
	   polydata->GetPolys()->GetNextCell(npts, pts); ) {
	fprintf(fp, "%d ", npts);
	for (pj=0; pj<npts; pj++)
	  fprintf(fp, "%d ", pts[pj]);
	fprintf(fp, "\n");
      }
      fprintf(fp, "\nPOINT_DATA %d\n", npt);
      fprintf(fp, "SCALARS scalars float\n");
      fprintf(fp, "LOOKUP_TABLE default\n");
      for (int si = 0; si < npt; si++) {
	float s = polydata->GetPointData()->GetScalars()->GetScalar(si);
	fprintf(fp, "%f ", s);
	if (!((si+1)%6)) fprintf(fp, "\n");
      }
      fprintf(fp, "\n");
      fclose(fp);
    }
    else {
      fprintf(stderr, "could not open %s\n", argv[2]);
    }
    polydata->Delete();
    return(0);
}
